; Slider by David Phillips <david@acz.org>
; for the TI-86 graphing calculator
; written on May 23, 2000

#include "ti86asm.inc"

GrayMem1        = $f800
GrayMem2        = $fc00
MapBuffer1      = $8100         ; [768]
MapBuffer2      = $8400         ; [768]
GrayBuffer1     = $8700         ; [1024]
GrayBuffer2     = $8b00         ; [1024]
Vars            = $8f00         ; [256]
Map             = $9000         ; [256]

ScreenPos       = Vars          ; [1]       x position of the screen relative to the map
Frame           = Vars + 1      ; [1]       counter incremented every frame
Timer           = Vars + 2      ; [1]       counter incremented every interrupt
Slider          = Vars + 3      ; [6]       y, x, dir(0 = up, 1 = down, 2 = left, 3 = right), anim(0..2), current tile
Tiles           = Vars + 9      ; [1]       number of tiles left for slider to turn
Lives           = Vars + 10     ; [1]       lives player has left
Time            = Vars + 11     ; [1]       time player has left to complete level
TextBuf         = Vars + 12     ; [6]       buffer for unpacking numbers
TileAnim        = Vars + 18     ; [1]       tile animation counter

.org _asm_exec_ram

 nop
 jp ProgEntry
 .dw 0
 .dw Title

Title:
 .db "Slider",0

ProgEntry:
 call _runindicoff
 call _flushallmenus
 call OpenGray

 call ClearMap
 ld hl,Map1
 call LoadMap

 sub a
 ld (Frame),a
 ld (TileAnim),a
 ld a,5
 ld (Lives),a
 

 call ClearBuffer

 ld hl,$2020
 ld (Slider),hl
 ld hl,$0101
 ld (Slider+2),hl

MainLoop:
 ld hl,Frame
 inc (hl)
 call CopyMap

 halt
 ld a,%01111110
 out (1),a
 nop
 nop
 in a,(1)
 rra
 call nc,MoveSliderDown
 rra
 call nc,MoveSliderLeft
 rra
 call nc,MoveSliderRight
 rra
 call nc,MoveSliderUp

 ld hl,TileAnim
 inc (hl)
 ld a,(hl)
 and %00000111
 call z,ChangeMap

 call CheckSliderTile

 ld hl,Slider
 ld de,SliderSprites
 call GrayLoadDrawSprite

 call ShowBuffer
 call DrawStatus

 ld hl,Timer
Delay:
 ld a,(hl)
DelayDiv:
 sub 6
 jr z,DelayDone
 jr nc,DelayDiv
 halt
 jr Delay
DelayDone:
 
 call _get_key
 cp K_ALPHA
 jr nz,NotAlpha
 ld bc,(Slider)
 call GetTileCent
 ld a,(de)
 cp 12
 jr nz,NotAlpha
 
NotAlpha:
 cp K_EXIT
 jr nz,MainLoop
 call CloseGray
 jp _clrWindow

ChangeMap:
 ld hl,Map
 ld c,224
ChangeMapTiles:
 ld de,ChangeList
 ld b,ChangeListSize
ChangeMapTile:
 ld a,(de)
 inc de
 cp (hl)
 jr nz,ChangeMapSkip
 ld a,(de)
 ld (hl),a
 push hl
 push de
 push bc
 call DrawTile
 pop bc
 pop de
 pop hl
 jr ChangeMapNext
ChangeMapSkip:
 inc de
 djnz ChangeMapTile
ChangeMapNext:
 inc hl
 dec c
 jr nz,ChangeMapTiles
 ret

ChangeList:
 .db 06,07, 07,08
 .db 10,11, 11,00
 .db 17,18, 18,19
 .db 21,22, 22,23
 .db 25,26, 26,27
 .db 29,30, 30,31
ChangeListSize = ($ - ChangeList) / 2
 

DrawStatus:
 ld bc,(1<<7)+12
 ld hl,StatusTextTile
 call PrintTextString
 ld bc,(3<<7)+12
 ld hl,StatusTextTime
 call PrintTextString
 ld bc,(5<<7)+12
 ld hl,StatusTextLeft
 call PrintTextString
 ld bc,(2<<7)+13
 ld a,(Tiles)
 call DispA
 ld bc,(4<<7)+13
 ld a,(Time)
 call DispA
 ld bc,(6<<7)+13
 ld a,(Lives)
 call DispA
 ret

StatusTextTile:
 .db "Tile",0

StatusTextTime:
 .db "Time",0

StatusTextLeft:
 .db "Left",0
 
; a = number to display, bc = offset on screen
DispA:
 push bc
 ld h,0
 ld l,a
 ld de,TextBuf+5
 sub a
 ld (de),a
 ld b,3
DispHLL:
 call _divHLby10
 add a,'0'
 dec de
 ld (de),a
 djnz DispHLL
 ex de,hl
 pop bc

; hl = string, bc = correct offset on screen
PrintTextString:
 ld a,(hl)
 inc hl 
 or a
 ret z
 push hl
 push bc
 call PrintChar
 pop bc
 inc bc
 pop hl
 jr PrintTextString

; a = char to print, bc = offset in buffer
PrintChar:
 add a,a
 ld l,a
 ld h,0
 add hl,hl
 add hl,hl
 ld ix,TextFont-(' '<<3)
 ex de,hl
 add ix,de
 ld hl,GrayBuffer1
 add hl,bc
 ex de,hl
 ld hl,GrayBuffer2
 add hl,bc
 ld bc,16
 ld a,8
PrintCharLoop:
 ld (_@PrintCharLoopC),a
 ld a,(ix+0)
 inc ix
 ld (hl),a
 add hl,bc
 ex de,hl
 ld (hl),a
 add hl,bc
 ex de,hl
_@PrintCharLoopC =$+1
 ld a,0
 dec a
 jr nz,PrintCharLoop
 ret

CheckSliderTile:
 ld bc,(Slider)
 call GetTileCent
 ld hl,(Slider+4)
 ld (Slider+4),de
 or a
 sbc hl,de
 push af
 ld a,(Frame)
 and %00000001
 jr nz,CheckSliderNotA
 ld hl,Slider+1
 ld a,(de)
 cp 16
 call z,CheckSliderTurn
 jr c,CheckSliderNotA
 cp 20
 call z,CheckSliderTurn
 jr nc,CheckSliderNotR
 call MoveSliderRightDo
 jr CheckSliderNotA
CheckSliderNotR:
 cp 24
 call z,CheckSliderTurn
 jr nc,CheckSliderNotD
 call MoveSliderDownDo
 jr CheckSliderNotA
CheckSliderNotD:
 cp 28
 call z,CheckSliderTurn
 jr nc,CheckSliderNotL
 call MoveSliderLeftDo
 jr CheckSliderNotA
CheckSliderNotL:
 cp 32
 jr nc,CheckSliderNotA
 call MoveSliderUpDo
CheckSliderNotA:
 pop af
 ret z
 ld a,(de)
 cp 5
 call z,CheckSliderTurn
 cp 9
 ret nz
; call z,CheckSliderTurn

; de = tile to turn, a = tile
CheckSliderTurn:
 push af
 push hl
 inc a
 ld (de),a
 ex de,hl
 call DrawTile
 ld hl,Tiles
 dec (hl)
 sub a
 ld (TileAnim),a
 pop hl
 pop af
 ret

; hl = tile to draw
DrawTile:
 ex de,hl
 ld a,(de)
 ld h,0
 ld l,a
 add hl,hl
 add hl,hl
 add hl,hl
 add hl,hl
 ld bc,MapTiles
 add hl,bc
 push hl
 ex de,hl
 ld de,Map
 sbc hl,de
 ld a,l
 ld b,l
 and %11100000
 ld l,a
 ld d,h
 ld e,l
 add hl,hl
 add hl,de
 ld a,(ScreenPos)
 rra
 rra
 rra
 and %00011111
 ld c,a
 ld a,b
 and %00011111
 sub c
 ret c
 cp 12
 ret nc
 ld c,a
 ld b,0
 add hl,bc
 ex de,hl
 ld a,(ScreenPos)
 and %00000111
 ld c,a
 ld a,8
 sub c
 pop hl
 push de
 push af
 ld ix,MapBuffer1
 add ix,de
 call DrawTileRender
 pop af
 pop de
 ld ix,MapBuffer2
 add ix,de

; ix = screen buffer, hl = sprite pointer, a = times to shift
; returns: hl = next sprite
DrawTileRender:
 ld (_@DrawTileRenderS),a
 ld b,8
DrawTileRenderL:
 push bc
 ld b,(hl)
 inc hl
 ld c,0
 ld de,$00ff
_@DrawTileRenderS =$+1
 ld a,0
 or a
 jr z,DrawTileRenderSD
DrawTileRenderS:
 srl b
 rr c
 scf
 rr d
 rr e
 dec a
 jr nz,DrawTileRenderS 
DrawTileRenderSD:
 ld a,(ix+0)
 and d
 or b
 ld (ix+0),a
 ld a,(ix+1)
 and e
 or c
 ld (ix+1),a
 ld bc,12
 add ix,bc
 pop bc
 djnz DrawTileRenderL
 ret

; hl = tile to convert
; returns: bc = x,y coords on screen
GetCoords:
 ld de,Map
 or a
 sbc hl,de
 ld a,l
 rra
 rra
 and %00111000
 add a,8
 ld c,a
 ld a,l
 and %00011111
 add a,a
 add a,a
 add a,a
 ld hl,ScreenPos
 sub (hl)
 ld b,a
 ret

MoveSliderDownDo:
 push af
 jr MoveSliderDownGo
MoveSliderDown:
 push af
 ld hl,Slider+3
 call UpdateAnim
 dec hl 
 ld (hl),1
 dec hl
MoveSliderDownGo:
 ld b,(hl)
 dec hl
 ld c,(hl)
 call TryMoveDown
 pop af
 ret

MoveSliderLeftDo:
 push af
 jr MoveSliderLeftGo
MoveSliderLeft:
 push af
 ld hl,Slider+3
 call UpdateAnim
 dec hl 
 ld (hl),2
 dec hl
MoveSliderLeftGo:
 ld b,(hl)
 dec hl
 ld c,(hl)
 inc hl
 call TryMoveLeft
 jr c,MoveSliderLeftDone
 ld a,(hl)
 cp 24
 jr nz,MoveSliderLeftDone
 push hl
 call ScrollLeft
 pop hl
 jr c,MoveSliderLeftDone
 inc (hl)
MoveSliderLeftDone:
 pop af
 ret

MoveSliderRightDo:
 push af
 jr MoveSliderRightGo
MoveSliderRight:
 push af
 ld hl,Slider+3
 call UpdateAnim
 dec hl 
 ld (hl),3
 dec hl
MoveSliderRightGo:
 ld b,(hl)
 dec hl
 ld c,(hl)
 inc hl
 call TryMoveRight
 jr c,MoveSliderRightDone
 ld a,(hl)
 cp 48
 jr nz,MoveSliderRightDone
 push hl
 call ScrollRight
 pop hl
 jr c,MoveSliderRightDone
 dec (hl)
MoveSliderRightDone:
 pop af
 ret

MoveSliderUpDo:
 push af
 jr MoveSliderUpGo
MoveSliderUp:
 push af
 ld hl,Slider+3
 call UpdateAnim
 dec hl 
 ld (hl),0
 dec hl
MoveSliderUpGo:
 ld b,(hl)
 dec hl
 ld c,(hl)
 call TryMoveUp
 pop af
 ret

; hl = pointer to map to load
LoadMap:
 ld de,Map
 ld bc,224
 ldir
 ld a,(hl)
 ld (Tiles),a
 inc hl
 ld a,(hl)
 ld (Time),a
 inc hl
 ld b,(hl)
 inc hl
 ld c,(hl)
 inc hl
 ld (Slider),bc
 call GetTile
 ld (Slider+4),de
 ld a,1
 ld (Slider+2),a 
 ld a,(hl)
 ld (ScreenPos),a
 rra
 rra
 rra
 and %00011111
 call DrawMap
 ret

ScrollLeft:
 ld hl,ScreenPos
 ld a,(hl)
 or a
 scf
 ret z
 dec (hl)
 ld hl,MapBuffer1
 ld b,0
ScrollLeftLoop:
 rr (hl)
 inc hl
 rr (hl)
 inc hl
 rr (hl)
 inc hl
 rr (hl)
 inc hl
 rr (hl)
 inc hl
 rr (hl)
 inc hl
 djnz ScrollLeftLoop
 ld hl,ScreenPos
 ld a,(hl)
 and %00000111
 ret nz
 ld a,(hl)
 rra
 rra
 rra
 and %00011111
 ld hl,Map
 ld de,MapBuffer1
 jp DrawColumn

ScrollRight:
 ld hl,ScreenPos
 ld a,(hl)
 cp 176
 scf
 ret z
 inc (hl)
 ld hl,MapBuffer2+767
 ld b,0
ScrollRightLoop
 rl (hl)
 dec hl
 rl (hl)
 dec hl
 rl (hl)
 dec hl
 rl (hl)
 dec hl
 rl (hl)
 dec hl
 rl (hl)
 dec hl
 djnz ScrollRightLoop 
 ld hl,ScreenPos
 ld a,(hl)
 and %00000111
 ret nz
 ld a,(hl)
 rra
 rra
 rra
 and %00011111
 add a,11
 ld hl,Map
 ld de,MapBuffer1+11
 jp DrawColumn

; hl = byte to update
UpdateAnim:
 ld a,(Frame)
 and %00000011
 ret nz
 inc (hl)
 ld a,3
 cp (hl)
 ret nz
 ld (hl),0
 ret 

; hl = address of x or y coord
; bc = coords loaded (x, y)
; returns: carry set = collision
TryMoveUp:
 ld a,c
 cp 9
 ret c
 dec c                  ; check tile above
 call CheckSpriteVert   ; check for collision
 ret c                  ; if we collided, return
 dec (hl)               ; move up
 ret                    ; return
TryMoveDown:
 ld a,c
 cp 120
 ccf
 ret c 
 ld a,8                 ; check tile below
 add a,c                ; sprites are 8 pixels
 ld c,a                 ; save coord
 call CheckSpriteVert   ; check for collision
 ret c                  ; if we collided, return
 inc (hl)               ; move down
 ret                    ; return
TryMoveLeft:
 dec b                  ; check tile to the left
 call CheckSpriteHorz   ; check for collision
 ret c                  ; if we collided, return
 dec (hl)               ; move left
 ret                    ; return
TryMoveRight:
 ld a,8                 ; check tile to the right
 add a,b                ; sprites are 8 pixels
 ld b,a                 ; save coord
 call CheckSpriteHorz   ; check for collision
 ret c                  ; if we collided, return
 inc (hl)               ; move right
 ret                    ; return

; check sprite and sprite to the right
; bc = sprite location to check (x, y)
; returns: carry = collision
CheckSpriteVert:
 call GetTile           ; calc address of tile on map
 call CheckSolid        ; is it solid?
 ret c
 ld a,(ScreenPos)
 add a,b                ; get x coord
 and %00000111          ; is it alligned?
 ret z                  ; then we're done
 inc de                 ; check sprite to the right
 call CheckSolid        ; is it solid?
 ret                    ; we're done

; check sprite and sprite to the bottom 
; bc = sprite location to check (x, y)
; returns: carry = collision
CheckSpriteHorz:
 call GetTile           ; calc address of tile on map
 call CheckSolid        ; is it solid?
 ret c                  ; then we're done
 ld a,c                 ; get y coord 
 and %00000111          ; is it alligned?
 ret z                  ; then we're done
 ex de,hl               ; do calculations in hl
 ld bc,32               ; get tile to the bottom
 add hl,bc              ; calc offset
 ex de,hl               ; swap back
 call CheckSolid        ; is it solid?
 ret                    ; we're done

; de = address of tile to check
; returns: carry = tile is solid
CheckSolid:
 ld a,(de)
 cp 1
 ret c
 cp 4
 ret c
 cp 15
 ccf
 ret z
 ret nc
 cp 16
 ret 

; hl = coords in (y, x) format
GetTileHL:
 ld c,(hl)              ; get y coord
 inc hl                 ; next byte
 ld b,(hl)              ; get x coord
 dec hl                 ; back to last byte 

; bc = coords to get (x, y)
; returns: de = pointer to map tile
GetTile:
 ex de,hl
 push bc
 ld a,c
 and %1111000
 sub 8
 add a,a
 add a,a
 ld h,0
 ld l,a
 ld a,(ScreenPos)
 add a,b
 ld b,0
 jr nc,GetTileNC
 inc b
GetTileNC:
 ld c,a
 srl b
 rr c
 srl b
 rr c
 srl b
 rr c
 add hl,bc
 ld bc,Map
 add hl,bc
 ex de,hl
 pop bc
 ret

; bc = coords to get (x, y)
; returns: de = pointer to map tile
GetTileCent:
 call CenterCoords
 jr GetTile 

; bc = coords to center (x, y)
; returns: bc = centered coords (x+3, y+3)
CenterCoords:
 inc b
 inc b
 inc b
 inc c
 inc c
 inc c
 ret

; bc = (x1,y1), de = (x2,y2)
; returns: no carry = collision
CheckCollision:
 ld a,d
 add a,7
 cp b
 ret c
 ld a,b
 add a,7
 cp d
 ret c
 ld a,e
 add a,7
 cp c
 ret c
 ld a,c
 add a,7
 cp e
 ret 

; in: bc = point, de = coords
; returns: no carry = collision
CheckPoint:
 ld a,d
 add a,7
 cp b
 ret c
 ld a,e
 add a,7
 cp c
 ret c
 ld a,b
 cp d
 ret c
 ld a,c
 cp e
 ret

; hl = coords (then dir, anim)
; de = start of sprites
GrayLoadDrawSprite:
 ld c,(hl)
 inc hl
 ld b,(hl)
 inc hl
; hl = dir byte (followed by anim byte)
; de = start of sprites
; bc = sprite draw location (x, y)
GrayDrawSprite:
 push bc                ; save draw location
 ld a,(hl)              ; load dir byte
 rlca                   ; each set is 72 bytes (* 2)
 rla                    ; * 4
 rla                    ; * 8
 ld c,a                 ; save (* 8)
 rla                    ; * 16
 rla                    ; * 32
 rla                    ; * 64
 add a,c                ; (* 64) + (* 8) = * 72
 ld c,a                 ; save (* 72)
 inc hl                 ; move to anim byte
 ld a,(hl)              ; get anim byte
 ld h,0                 ; clear high byte
 rla                    ; sprites are 24 bytes (* 2)
 rla                    ; * 4
 rla                    ; * 8
 ld b,a                 ; save (* 8)
 rla                    ; * 16
 add a,b                ; (* 16) + (* 8) = * 24
 add a,c                ; calc total offset
 jr nc,GrayDrawSpriteNC ; skip carry
 inc h                  ; increase high byte
GrayDrawSpriteNC:
 ld l,a                 ; load into low byte
 add hl,de              ; calculate correct sprite
 pop bc                 ; restore draw location
 jp GraySpriteClip      ; draw sprite and return

ClearBuffer:
 ld hl,GrayBuffer1
 ld de,GrayBuffer1+1
 ld bc,2047
 ld (hl),0
 ldir
 ret

ShowBuffer:
 ld hl,GrayBuffer1
 ld de,GrayMem1
 ld bc,2048
 ldir
 ret

CopyMap:
 ld hl,MapBuffer1+1
 ld de,GrayBuffer1+128
 call CopyMapDo
 ld hl,MapBuffer2+1
 ld de,GrayBuffer2+128
CopyMapDo:
 ld a,56
CopyMapLoop:
 ld bc,10
 ldir
 ex de,hl
 ld bc,6
 add hl,bc
 ex de,hl
 inc hl
 inc hl
 dec a
 jr nz,CopyMapLoop
 ret

ClearMap:
 ld hl,MapBuffer1
 ld de,MapBuffer1+1
 ld bc,1536
 ld (hl),0
 ldir
 ret

; hl = pointer to map, a = column, de = map buffer
DrawColumn: 
 dec a
 ld b,0
 ld c,a
 add hl,bc
 push hl
 pop ix
 ld b,7
DrawColumnLoop:
 call MapDrawDo
 ex de,hl
 ld de,31
 add ix,de
 ld de,95
 add hl,de
 ex de,hl
 djnz DrawColumnLoop
 ret

; a = start column
DrawMap:
 ld hl,Map-1
 ld b,0
 ld c,a
 add hl,bc
 push hl
 pop ix
 ld de,MapBuffer1
 ld c,7
DrawMapCol:
 ld b,12
DrawMapRow:
 call MapDrawDo
 djnz DrawMapRow
 ex de,hl
 ld de,20
 add ix,de
 ld de,84
 add hl,de
 ex de,hl
 dec c
 jr nz,DrawMapCol
 ret

DrawMapGSprite:
 call DrawMapSprite
 ex de,hl
 ld bc,767
 add hl,bc
 ex de,hl
 call DrawMapSprite
 ex de,hl
 ld bc,-768
 add hl,bc
 ex de,hl
 ret

MapDrawDo:
 ld a,(ix+0)
 inc ix
 ld h,0
 ld l,a
 add hl,hl
 add hl,hl
 add hl,hl
 add hl,hl
 push bc
 ld bc,MapTiles
 add hl,bc
 call DrawMapGSprite
 pop bc
 ret

; hl = sprite, de = buffer location
; returns: hl = next sprite, de = next buffer location
DrawMapSprite:
 ld a,8
 ld bc,11
DrawMapSpriteL:
 ldi
 inc bc
 ex de,hl
 add hl,bc
 ex de,hl
 dec a
 jr nz,DrawMapSpriteL
 ex de,hl
 ld bc,-95
 add hl,bc
 ex de,hl
 ret 


GrayDraw1 = GrayBuffer1             ; [1024] dark grayscale plane
GrayDraw2 = GrayBuffer2             ; [1024] light grayscale plane

; in: hl = 8x8 masked grayscale sprite, bc = x,y location
GraySpriteClip:
 ex de,hl                           ; save pointer in de
 ld hl,GraySpriteE                  ; need to call graysprite
 ld (_@ClipSpriteE),hl              ; modify call address

; in: de = sprite pointer, bc = x,y location
ClipSprite:
 ld a,b                             ; get x coord
 cp 249                             ; hanging off left edge?
 jr nc,ClipSpriteL                  ; then clip left edge
 cp 128                             ; completely off right edge?
 ret nc                             ; then don't draw it
 jr ClipSpriteR                     ; clip right edge
ClipSpriteL:
 res 7,b                            ; draw on right edge
 dec c                              ; move up a row (left clipping)
 bit 7,c                            ; are we on the top edge?
 jr z,ClipSpriteA                   ; if not, skip top fix
 ld hl,_@RenderSpriteCV             ; point to rows to draw
 inc (hl)                           ; increase it (decreased later)
 dec de                             ; decrease sprite pointer
 ld hl,-16                          ; need to draw up a row
 ld (_@ClipSpriteX),hl              ; modify top row fix value
ClipSpriteA:
 ld hl,$0218                        ; change instruction to <jr 2>
 ld (_@RenderSpriteCL),hl           ; modify to skip drawing left byte
 jr ClipSpriteH                     ; done clipping horizontal
ClipSpriteR:
 cp 120                             ; hanging off right edge?
 jr c,ClipSpriteH                   ; if not, don't clip
 ld hl,$0218                        ; change instruction to <jr 2>
 ld (_@RenderSpriteCR),hl           ; modify to skip drawing right byte
ClipSpriteH:
 ld a,c                             ; get y coord
 cp 249                             ; hanging off top?
 jr nc,ClipSpriteT                  ; then clip the top
 cp 63                              ; completely off the bottom?
 jr nc,ClipSpriteF                  ; then don't draw, but fix everything
 jr ClipSpriteB                     ; clip the bottom
ClipSpriteT:
 xor a                              ; clear for subtract
 ld h,a                             ; clear high byte
 sub c                              ; calc bytes above screen
 ld c,h                             ; set y coord to 0
 ld l,a                             ; set low byte to bytes above screen
 add hl,de                          ; advance sprite pointer
 ex de,hl                           ; swap back into de
 jr ClipSpriteV                     ; finish vertical clipping
ClipSpriteB:
 sub 56                             ; calc bytes below screen
 jr c,ClipSpriteD                   ; don't clip if not below screen
; inc a                             ; increase bytes below screen
ClipSpriteV:
 neg                                ; negate so it can be subtracted
_@ClipSpriteZ =$+1
 ld hl,_@RenderSpriteCV             ; point to rows to draw
 add a,(hl)                         ; add (subtract) from current value
 ld (hl),a                          ; save new value
ClipSpriteD:
 push de                            ; push sprite pointer
 pop ix                             ; pop sprite pointer in ix
 call FindAddress                   ; calc address and offset of sprite
_@ClipSpriteX =$+1
 ld de,0                            ; load address fix for top row
 add hl,de                          ; add the fix value
_@ClipSpriteE =$+1
 call $ffff                         ; call the correct sprite routine
ClipSpriteF:
 ld a,8                             ; normally 8 rows are drawn
 ld (_@RenderSpriteCV),a            ; set rows to draw back
 ld hl,$a678                        ; normal opcode is <ld a,b \ and (hl)>
 ld (_@RenderSpriteCL),hl           ; set instructions back
 inc l                              ; change <ld a,b> to <ld a,c>
 ld (_@RenderSpriteCR),hl           ; set instructions back
 ld hl,0                            ; normally top row fix is 0
 ld (_@ClipSpriteX),hl              ; set rox fix back
 ret                                ; return from clipping

; in: hl = 8x8 masked grayscale sprite, bc = x,y location
GraySprite:
 push hl                            ; push sprite pointer
 pop ix                             ; pop in ix
 call FindAddress                   ; calc address and offset
GraySpriteE:
 push hl                            ; save address offset
 push ix                            ; save sprite pointer
 ld de,GrayDraw1                    ; draw to first gray buffer
 add hl,de                          ; add offset to buffer start
 ld a,16                            ; grayscale sprites are 16 bytes
 ld (_@RenderSpriteM),a             ; adjust sprite load instruction
 call RenderSprite                  ; draw the sprite
 ld a,8                             ; second sprite is 8 bytes from mask
 ld (_@RenderSpriteM),a             ; adjust sprite load instruction back
 pop ix                             ; restore sprite pointer
 pop hl                             ; restore address offset
 ld de,GrayDraw2                    ; now draw to second gray buffer
 add hl,de                          ; add offset to second buffer start
 ld de,$0008                        ; sprites are 8 bytes
 add ix,de                          ; advance sprite pointer

; in: hl = address in buffer, c = offset, ix = sprite
RenderSprite:
_@RenderSpriteCV =$+1
 ld b,8                             ; set row counter to 8 bytes
RenderSpriteL:
 push bc                            ; save row counter and sprite offset
 ld a,(ix+8)                        ; load mask for this row
_@RenderSpriteM =$-1
 cpl                                ; invert the mask
 ld b,a                             ; save inverted mask
 ld d,(ix)                          ; load sprite byte for this row
 inc ix                             ; advance sprite pointer
 ld a,c                             ; set shift counter to sprite offset
 ld c,$ff                           ; set all bits of right mask byte
 ld e,0                             ; clear right byte of sprite
 or a                               ; is the sprite aligned?
 jr z,RenderSpriteD                 ; then skip shifting it
RenderSpriteS:
 srl d                              ; shift left sprite byte
 rr e                               ; into the right sprite byte
 scf                                ; set carry
 rr b                               ; shift left mask byte
 rr c                               ; into the right mask byte
 dec a                              ; decrease shift counter
 jr nz,RenderSpriteS                ; loop until counter is 0
RenderSpriteD:
_@RenderSpriteCL =$
 ld a,b                             ; load the left sprite byte
 and (hl)                           ; and it with the screen
 or d                               ; or it with the mask
 ld (hl),a                          ; set screen memory to left byte
 inc hl                             ; increase screen pointer
_@RenderSpriteCR =$
 ld a,c                             ; load the right sprite byte
 and (hl)                           ; and it with the screen
 or e                               ; or it with the mask
 ld (hl),a                          ; set screen memory to left byte
 ld de,15                           ; next row is 15 bytes ahead
 add hl,de                          ; move screen pointer to next row
 pop bc                             ; restore loop counter and shift mask
 djnz RenderSpriteL                 ; loop for all bytes in sprite
 ret                                ; done drawing the sprite

; in: bc = x,y location
; out: hl = offset of byte, c = offset in video byte
FindAddress:
 ld a,c                             ; get the y coord
 add a,a                            ; multiply by 2
 add a,a                            ; again by 4
 ld l,a                             ; save as low byte
 ld h,0                             ; clear high byte
 add hl,hl                          ; multiply by 8
 add hl,hl                          ; and again by 16
 ld a,b                             ; now get the x coord
 and %11111000                      ; clear lower 3 bits
 rra                                ; divide by 2
 rra                                ; this time by 4
 rra                                ; and finally by 8
 or l                               ; combine with low byte
 ld l,a                             ; save it
 ld a,b                             ; get x coord again
 and %00000111                      ; clear all but last 3 bits
 ld c,a                             ; save sprite offset in c
 ret                                ; return to caller

OpenGray:
 di
OpenGrayMoveStack:
 ld hl,-$400
 add hl,sp
 ld sp,hl
 ld hl,$fb00
 ld de,$f700
 ld bc,256
 ldir
OpenGrayInstall:
 ld hl,IntStart
 ld de,$D2FE
 push de
 ld bc,IntEnd-IntStart
 ldir
 ld de,$28
 pop hl
 ld a,(hl)
 dec hl
 push hl
 ld b,5
OpenGrayCalcChecksum:
 add hl,de
 add a,(hl)
 djnz OpenGrayCalcChecksum
 pop hl
 ld (hl),a
 set 2,(iy+$23)
 exx
 ld bc,$3800
 ld de,$0304
 exx
 ld hl,($4065)
 ld (IntROMpatch),hl
 sub a
 ld (Timer),a
 ei
 
ClearGray:
 ld hl,GrayMem1
 ld de,GrayMem1+1
 ld bc,2047
 ld (hl),0
 ldir
 ret

CloseGray:
 res 2,(iy+$23)
 ld a,$3c
 out (0),a
CloseGrayRestoreStack:
 di
 ld hl,$400
 add hl,sp
 ld sp,hl
 ld hl,$f700
 ld de,$fb00
 ld bc,256
 ldir
 ei
 ret

IntStart:
 in a,(3)
 bit 1,a
 jr z,IntExit
 out (c),b
 dec d
 jr nz,IntSkip
 ld d,3
 jr IntSwap
IntSkip:
 ld a,d
 dec a
 jr nz,IntExit
IntSwap:
 ld a,b
 xor e
 ld b,a
IntExit:
 pop hl
 ld hl,Timer
 inc (hl)
 ld a,(hl)
 cp 218
 jr nz,IntTimeDecSkip
 ld (hl),0
 ld hl,Time
 dec (hl)
IntTimeDecSkip:
 in a,(3)
 rra
 push af
 push bc
 push de
 call nc,$01A1
IntROMpatch = $-2
 pop de
 pop bc
 pop af
 ld a,9
 adc a,0
 out (3),a
 ld a,11
 out (3),a
 ex af,af'
 exx
 ei
 reti
IntEnd:


SliderSprites:

SliderSprUp:
    .db %00111100
    .db %01111110
    .db %01111110
    .db %01111110
    .db %11111111
    .db %11111111
    .db %00111100
    .db %01100110
    .db %00000000
    .db %00000000
    .db %00000000
    .db %00000000
    .db %11001011
    .db %10011001
    .db %10000001
    .db %01100110
    .db %00111100
    .db %01111110
    .db %01111110
    .db %01111110
    .db %11111111
    .db %11111111
    .db %10111101
    .db %01100110

SliderSprUp2:
    .db %00111100
    .db %01111110
    .db %01111110
    .db %01111111
    .db %11111111
    .db %11111110
    .db %00111100
    .db %00000110
    .db %00000000
    .db %00000000
    .db %00000001
    .db %00000001
    .db %11010011
    .db %10011000
    .db %10000000
    .db %00000110
    .db %00111100
    .db %01111110
    .db %01111111
    .db %01111111
    .db %11111111
    .db %11111110
    .db %10111100
    .db %00000110

SliderSprUp3:
    .db %00111100
    .db %01111110
    .db %01111110
    .db %11111110
    .db %11111111
    .db %01111111
    .db %00111100
    .db %01100000
    .db %00000000
    .db %00000000
    .db %10000000
    .db %10000000
    .db %11001011
    .db %00011001
    .db %00000001
    .db %01100000
    .db %00111100
    .db %01111110
    .db %11111110
    .db %11111110
    .db %11111111
    .db %01111111
    .db %00111101
    .db %01100000

SliderSprDown:
    .db %00111100
    .db %01111110
    .db %01100110
    .db %11000010
    .db %11011011
    .db %01000011
    .db %00111100
    .db %01100000
    .db %00000000
    .db %00000000
    .db %10111100
    .db %10111100
    .db %11011011
    .db %00000001
    .db %00000001
    .db %01100000
    .db %00111100
    .db %01111110
    .db %11111110
    .db %11111110
    .db %11111111
    .db %01111111
    .db %00111101
    .db %01100000

SliderSprDown2:
    .db %00111100
    .db %01111110
    .db %01100110
    .db %01000011
    .db %11011011
    .db %11000010
    .db %00111100
    .db %00000110
    .db %00000000
    .db %00000000
    .db %00111101
    .db %00111101
    .db %11011011
    .db %10000000
    .db %10000000
    .db %00000110
    .db %00111100
    .db %01111110
    .db %01111111
    .db %01111111
    .db %11111111
    .db %11111110
    .db %10111100
    .db %00000110

SliderSprDown3:
    .db %00111100
    .db %01111110
    .db %01100110
    .db %01000011
    .db %11011011
    .db %11000010
    .db %00111100
    .db %00000110
    .db %00000000
    .db %00000000
    .db %00111101
    .db %00111101
    .db %11011011
    .db %10000000
    .db %10000000
    .db %00000110
    .db %00111100
    .db %01111110
    .db %01111111
    .db %01111111
    .db %11111111
    .db %11111110
    .db %10111100
    .db %00000110

SliderSprLeft:
    .db %01111100
    .db %00001110
    .db %01101110
    .db %00001110
    .db %11001110
    .db %01011111
    .db %00111110
    .db %01100110
    .db %00000000
    .db %01110000
    .db %01110000
    .db %01100000
    .db %10001000
    .db %00111001
    .db %00000000
    .db %01100110
    .db %01111100
    .db %01111110
    .db %01111110
    .db %01111110
    .db %11111110
    .db %01111111
    .db %00111110
    .db %01100110

SliderSprLeft2:
    .db %01111100
    .db %00001110
    .db %01101110
    .db %00001110
    .db %11001110
    .db %01111111
    .db %00011110
    .db %00110110
    .db %00000000
    .db %01110000
    .db %01110000
    .db %01100000
    .db %10001000
    .db %00001001
    .db %00110000
    .db %00110110
    .db %01111100
    .db %01111110
    .db %01111110
    .db %01111110
    .db %11111110
    .db %01111111
    .db %00111110
    .db %00110110

SliderSprLeft3:
    .db %01111100
    .db %00001110
    .db %01101110
    .db %00001110
    .db %11001110
    .db %01111111
    .db %00111110
    .db %01100011
    .db %00000000
    .db %01110000
    .db %01110000
    .db %01100000
    .db %10001000
    .db %00001001
    .db %00001000
    .db %01101011
    .db %01111100
    .db %01111110
    .db %01111110
    .db %01111110
    .db %11111110
    .db %01111111
    .db %00111110
    .db %01101011

SliderSprRight:
    .db %00111110
    .db %01110000
    .db %01110110
    .db %01110000
    .db %01110011
    .db %11111010
    .db %01111100
    .db %01100110
    .db %00000000
    .db %00001110
    .db %00001110
    .db %00000110
    .db %00010001
    .db %10011100
    .db %00000000
    .db %01100110
    .db %00111110
    .db %01111110
    .db %01111110
    .db %01111110
    .db %01111111
    .db %11111110
    .db %01111100
    .db %01100110

SliderSprRight2:
    .db %00111110
    .db %01110000
    .db %01110110
    .db %01110000
    .db %01110011
    .db %11111110
    .db %01111000
    .db %01101100
    .db %00000000
    .db %00001110
    .db %00001110
    .db %00000110
    .db %00010001
    .db %10010000
    .db %00001100
    .db %01101100
    .db %00111110
    .db %01111110
    .db %01111110
    .db %01111110
    .db %01111111
    .db %11111110
    .db %01111100
    .db %01101100

SliderSprRight3:
    .db %00111110
    .db %01110000
    .db %01110110
    .db %01110000
    .db %01110011
    .db %11111110
    .db %01111100
    .db %11000110
    .db %00000000
    .db %00001110
    .db %00001110
    .db %00000110
    .db %00010001
    .db %10010000
    .db %00010000
    .db %11010110
    .db %00111110
    .db %01111110
    .db %01111110
    .db %01111110
    .db %01111111
    .db %11111110
    .db %01111100
    .db %11010110


MapTiles:

Tile00:
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111

Tile01:
    .db %00111100
    .db %01111110
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %01111110
    .db %00111100
    .db %11111111
    .db %11111111
    .db %11000011
    .db %11011011
    .db %11011000
    .db %11000001
    .db %11110001
    .db %11110011

Tile02:
    .db %11111111
    .db %11000011
    .db %10100101
    .db %10011001
    .db %10011001
    .db %10100101
    .db %11000011
    .db %11111111
    .db %00000000
    .db %00000000
    .db %00000010
    .db %00000110
    .db %00000110
    .db %00011010
    .db %00111100
    .db %00000000

Tile03:
    .db %11111111
    .db %11000011
    .db %10100111
    .db %10011111
    .db %10011111
    .db %10111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111101
    .db %11111001
    .db %11111001
    .db %11100101
    .db %11000011
    .db %11111111

Tile04:
    .db %11111111
    .db %10101011
    .db %11010101
    .db %10101011
    .db %11010101
    .db %10101011
    .db %11010101
    .db %11111111
    .db %00000000
    .db %01010100
    .db %00101010
    .db %01010100
    .db %00101010
    .db %01010100
    .db %00101010
    .db %00000000

Tile05:
    .db %00000000
    .db %01000010
    .db %00000000
    .db %01000010
    .db %00000000
    .db %00001100
    .db %01000000
    .db %00000000
    .db %11111111
    .db %10111101
    .db %11111111
    .db %10111101
    .db %11111111
    .db %11110011
    .db %10111111
    .db %11111111

Tile06:
    .db %00100100
    .db %01001011
    .db %10010010
    .db %01100110
    .db %01001001
    .db %10011110
    .db %01100100
    .db %01001001
    .db %11011011
    .db %11110110
    .db %01101101
    .db %11011011
    .db %10110110
    .db %01101101
    .db %11011011
    .db %10110110

Tile07:
    .db %01010101
    .db %11101010
    .db %01010101
    .db %11101010
    .db %01010101
    .db %10101110
    .db %01010101
    .db %10101010
    .db %10101010
    .db %01010111
    .db %10101010
    .db %01010111
    .db %10101010
    .db %01011101
    .db %11101010
    .db %01010101

Tile08:
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %00000000
    .db %01000010
    .db %00000000
    .db %01000010
    .db %00000000
    .db %00001100
    .db %01000000
    .db %00000000

Tile09:
    .db %00000000
    .db %01100110
    .db %00101000
    .db %01010010
    .db %00001000
    .db %00001100
    .db %01010110
    .db %01100000
    .db %11111111
    .db %10111111
    .db %11111111
    .db %10111101
    .db %11111111
    .db %11111011
    .db %10111111
    .db %11111111

Tile10:
    .db %01010101
    .db %11101010
    .db %01010101
    .db %11101010
    .db %01010101
    .db %10101110
    .db %01010101
    .db %10101010
    .db %11111111
    .db %10111111
    .db %11111111
    .db %10111111
    .db %11111111
    .db %11111011
    .db %11111111
    .db %11111111

Tile11:
    .db %01101111
    .db %11111011
    .db %10111111
    .db %11101111
    .db %11111111
    .db %11110111
    .db %01111101
    .db %11011111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111

Tile12:
    .db %00000000
    .db %01111110
    .db %01111110
    .db %00011000
    .db %00011000
    .db %01111110
    .db %00111100
    .db %00011000
    .db %11111111
    .db %10000001
    .db %11100111
    .db %11100111
    .db %11100111
    .db %11000011
    .db %11100111
    .db %11111111

Tile13:
    .db %00000000
    .db %00111100
    .db %01111111
    .db %01111111
    .db %01111111
    .db %01111111
    .db %01111110
    .db %00111100
    .db %00000000
    .db %00111100
    .db %01111110
    .db %01111110
    .db %01111110
    .db %01111110
    .db %00111100
    .db %00000000

Tile14:
    .db %00000000
    .db %00000000
    .db %00000000
    .db %00000000
    .db %00000000
    .db %00000000
    .db %00000000
    .db %00000000
    .db %00000000
    .db %01010100
    .db %00101010
    .db %01010100
    .db %00101010
    .db %01010100
    .db %00101010
    .db %00000000

Tile15:
    .db %00111100
    .db %01111110
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %01111110
    .db %00111100
    .db %11000011
    .db %10000001
    .db %00111100
    .db %00100100
    .db %00100111
    .db %00111111
    .db %10001111
    .db %11001111

Tile16:
    .db %10000001
    .db %00001000
    .db %00001100
    .db %01111110
    .db %01111110
    .db %00001100
    .db %00001000
    .db %10000001
    .db %01111110
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %01111110

Tile17:
    .db %11001001
    .db %10011010
    .db %00101100
    .db %01111111
    .db %11111110
    .db %00101100
    .db %01001001
    .db %10010011
    .db %00110110
    .db %01101101
    .db %11011111
    .db %11111110
    .db %01111111
    .db %11011111
    .db %10111110
    .db %01101100

Tile18:
    .db %01101101
    .db %11011011
    .db %10111110
    .db %01111111
    .db %11111111
    .db %10111110
    .db %01101101
    .db %11011011
    .db %10010010
    .db %00101100
    .db %01001101
    .db %11111110
    .db %01111110
    .db %01001101
    .db %10011010
    .db %00100100

Tile19:
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %00000000
    .db %00001000
    .db %00001100
    .db %01111110
    .db %01111110
    .db %00001100
    .db %00001000
    .db %00000000

Tile20:
    .db %10000001
    .db %00011000
    .db %00011000
    .db %00011000
    .db %01111110
    .db %00111100
    .db %00011000
    .db %10000001
    .db %01111110
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %01111110

Tile21:
    .db %10010011
    .db %01011001
    .db %00111100
    .db %10011010
    .db %01111111
    .db %00111100
    .db %10011010
    .db %11001001
    .db %01101100
    .db %10111110
    .db %11011011
    .db %01111101
    .db %11111110
    .db %11111111
    .db %01111101
    .db %00110110

Tile22:
    .db %10110110
    .db %11011011
    .db %01111101
    .db %10111110
    .db %11111111
    .db %01111101
    .db %10111110
    .db %11011011
    .db %01001001
    .db %00111100
    .db %10011010
    .db %01011001
    .db %01111110
    .db %10111110
    .db %01011001
    .db %00100100

Tile23:
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %00000000
    .db %00011000
    .db %00011000
    .db %00011000
    .db %01111110
    .db %00111100
    .db %00011000
    .db %00000000

Tile24:
    .db %10000001
    .db %00010000
    .db %00110000
    .db %01111110
    .db %01111110
    .db %00110000
    .db %00010000
    .db %10000001
    .db %01111110
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %01111110

Tile25:
    .db %10010011
    .db %01011001
    .db %00110100
    .db %11111110
    .db %01111111
    .db %00110100
    .db %10010010
    .db %11001001
    .db %01101100
    .db %10110110
    .db %11111011
    .db %01111111
    .db %11111110
    .db %11111011
    .db %01111101
    .db %00110110

Tile26:
    .db %10110110
    .db %11011011
    .db %01111101
    .db %11111110
    .db %11111111
    .db %01111101
    .db %10110110
    .db %11011011
    .db %01001001
    .db %00110100
    .db %10110010
    .db %01111111
    .db %01111110
    .db %10110010
    .db %01011001
    .db %00100100

Tile27:
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %00000000
    .db %00010000
    .db %00110000
    .db %01111110
    .db %01111110
    .db %00110000
    .db %00010000
    .db %00000000

Tile28:
    .db %10000001
    .db %00011000
    .db %00111100
    .db %01111110
    .db %00011000
    .db %00011000
    .db %00011000
    .db %10000001
    .db %01111110
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %01111110

Tile29:
    .db %10010011
    .db %01011001
    .db %00111100
    .db %11111110
    .db %01011001
    .db %00111100
    .db %10011010
    .db %11001001
    .db %01101100
    .db %10111110
    .db %11111111
    .db %01111111
    .db %10111110
    .db %11011011
    .db %01111101
    .db %00110110

Tile30:
    .db %11011011
    .db %01111101
    .db %10111110
    .db %11111111
    .db %01111101
    .db %10111110
    .db %11011011
    .db %01101101
    .db %00100100
    .db %10011010
    .db %01111101
    .db %01111110
    .db %10011010
    .db %01011001
    .db %00111100
    .db %10010010

Tile31:
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %11111111
    .db %00000000
    .db %00011000
    .db %00111100
    .db %01111110
    .db %00011000
    .db %00011000
    .db %00011000
    .db %00000000

Tile32:
    .db %10011001
    .db %00111100
    .db %01111110
    .db %11111111
    .db %11111111
    .db %01111110
    .db %00111100
    .db %10011001
    .db %11111111
    .db %11111111
    .db %11011011
    .db %11100111
    .db %11100111
    .db %11011011
    .db %11111111
    .db %11111111

Tile33:
    .db %10011001
    .db %00111100
    .db %01111110
    .db %11111111
    .db %11111111
    .db %01111110
    .db %00111100
    .db %10011001
    .db %01100110
    .db %11000011
    .db %10100101
    .db %00011000
    .db %00011000
    .db %10100101
    .db %11000011
    .db %01100110

TextFont:

FontChar_20:
 .db %00000000
 .db %00000000
 .db %00000000
 .db %00000000
 .db %00000000
 .db %00000000
 .db %00000000
 .db %00000000

FontChar_21:
 .db %00011000
 .db %00111100
 .db %00111100
 .db %00011000
 .db %00011000
 .db %00000000
 .db %00011000
 .db %00000000

FontChar_22:
 .db %00110110
 .db %00110110
 .db %00110110
 .db %00000000
 .db %00000000
 .db %00000000
 .db %00000000
 .db %00000000

FontChar_23:
 .db %00110110
 .db %00110110
 .db %01111111
 .db %00110110
 .db %01111111
 .db %00110110
 .db %00110110
 .db %00000000

FontChar_24:
 .db %00011000
 .db %00111110
 .db %01100000
 .db %00111100
 .db %00000110
 .db %01111100
 .db %00011000
 .db %00000000

FontChar_25:
 .db %00000000
 .db %01100011
 .db %01100110
 .db %00001100
 .db %00011000
 .db %00110011
 .db %01100011
 .db %00000000

FontChar_26:
 .db %00011100
 .db %00110110
 .db %00011100
 .db %00111011
 .db %01101110
 .db %01100110
 .db %00111011
 .db %00000000

FontChar_27:
 .db %00011000
 .db %00011000
 .db %00110000
 .db %00000000
 .db %00000000
 .db %00000000
 .db %00000000
 .db %00000000

FontChar_28:
 .db %00001100
 .db %00011000
 .db %00110000
 .db %00110000
 .db %00110000
 .db %00011000
 .db %00001100
 .db %00000000

FontChar_29:
 .db %00110000
 .db %00011000
 .db %00001100
 .db %00001100
 .db %00001100
 .db %00011000
 .db %00110000
 .db %00000000

FontChar_2A:
 .db %00000000
 .db %01100110
 .db %00111100
 .db %11111111
 .db %00111100
 .db %01100110
 .db %00000000
 .db %00000000

FontChar_2B:
 .db %00000000
 .db %00011000
 .db %00011000
 .db %01111110
 .db %00011000
 .db %00011000
 .db %00000000
 .db %00000000

FontChar_2C:
 .db %00000000
 .db %00000000
 .db %00000000
 .db %00000000
 .db %00000000
 .db %00011000
 .db %00011000
 .db %00110000

FontChar_2D:
 .db %00000000
 .db %00000000
 .db %00000000
 .db %01111110
 .db %00000000
 .db %00000000
 .db %00000000
 .db %00000000

FontChar_2E:
 .db %00000000
 .db %00000000
 .db %00000000
 .db %00000000
 .db %00000000
 .db %00011000
 .db %00011000
 .db %00000000

FontChar_2F:
 .db %00000011
 .db %00000110
 .db %00001100
 .db %00011000
 .db %00110000
 .db %01100000
 .db %11000000
 .db %00000000

FontChar_30:
 .db %00111110
 .db %01100011
 .db %01100111
 .db %01101111
 .db %01111011
 .db %01110011
 .db %00111110
 .db %00000000

FontChar_31:
 .db %00011000
 .db %00111000
 .db %00011000
 .db %00011000
 .db %00011000
 .db %00011000
 .db %01111110
 .db %00000000

FontChar_32:
 .db %00111100
 .db %01100110
 .db %00000110
 .db %00011100
 .db %00110000
 .db %01100110
 .db %01111110
 .db %00000000

FontChar_33:
 .db %00111100
 .db %01100110
 .db %00000110
 .db %00011100
 .db %00000110
 .db %01100110
 .db %00111100
 .db %00000000

FontChar_34:
 .db %00001110
 .db %00011110
 .db %00110110
 .db %01100110
 .db %01111111
 .db %00000110
 .db %00001111
 .db %00000000

FontChar_35:
 .db %01111110
 .db %01100000
 .db %01111100
 .db %00000110
 .db %00000110
 .db %01100110
 .db %00111100
 .db %00000000

FontChar_36:
 .db %00011100
 .db %00110000
 .db %01100000
 .db %01111100
 .db %01100110
 .db %01100110
 .db %00111100
 .db %00000000

FontChar_37:
 .db %01111110
 .db %01100110
 .db %00000110
 .db %00001100
 .db %00011000
 .db %00011000
 .db %00011000
 .db %00000000

FontChar_38:
 .db %00111100
 .db %01100110
 .db %01100110
 .db %00111100
 .db %01100110
 .db %01100110
 .db %00111100
 .db %00000000

FontChar_39:
 .db %00111100
 .db %01100110
 .db %01100110
 .db %00111110
 .db %00000110
 .db %00001100
 .db %00111000
 .db %00000000

FontChar_3A:
 .db %00000000
 .db %00011000
 .db %00011000
 .db %00000000
 .db %00000000
 .db %00011000
 .db %00011000
 .db %00000000

FontChar_3B:
 .db %00000000
 .db %00011000
 .db %00011000
 .db %00000000
 .db %00000000
 .db %00011000
 .db %00011000
 .db %00110000

FontChar_3C:
 .db %00000110
 .db %00001100
 .db %00011000
 .db %00110000
 .db %00011000
 .db %00001100
 .db %00000110
 .db %00000000

FontChar_3D:
 .db %00000000
 .db %00000000
 .db %01111110
 .db %00000000
 .db %01111110
 .db %00000000
 .db %00000000
 .db %00000000

FontChar_3E:
 .db %00110000
 .db %00011000
 .db %00001100
 .db %00000110
 .db %00001100
 .db %00011000
 .db %00110000
 .db %00000000

FontChar_3F:
 .db %00111100
 .db %01100110
 .db %00000110
 .db %00001100
 .db %00011000
 .db %00000000
 .db %00011000
 .db %00000000

FontChar_40:
 .db %00111110
 .db %01100011
 .db %01101111
 .db %01101111
 .db %01101111
 .db %01100000
 .db %00111100
 .db %00000000

FontChar_41:
 .db %00011100
 .db %00110110
 .db %01100011
 .db %01100011
 .db %01111111
 .db %01100011
 .db %01100011
 .db %00000000

FontChar_42:
 .db %01111110
 .db %00110011
 .db %00110011
 .db %00111110
 .db %00110011
 .db %00110011
 .db %01111110
 .db %00000000

FontChar_43:
 .db %00011110
 .db %00110011
 .db %01100000
 .db %01100000
 .db %01100000
 .db %00110011
 .db %00011110
 .db %00000000

FontChar_44:
 .db %01111100
 .db %00110110
 .db %00110011
 .db %00110011
 .db %00110011
 .db %00110110
 .db %01111100
 .db %00000000

FontChar_45:
 .db %01111111
 .db %00110001
 .db %00110100
 .db %00111100
 .db %00110100
 .db %00110001
 .db %01111111
 .db %00000000

FontChar_46:
 .db %01111111
 .db %00110001
 .db %00110100
 .db %00111100
 .db %00110100
 .db %00110000
 .db %01111000
 .db %00000000

FontChar_47:
 .db %00011110
 .db %00110011
 .db %01100000
 .db %01100000
 .db %01100111
 .db %00110011
 .db %00011111
 .db %00000000

FontChar_48:
 .db %01100110
 .db %01100110
 .db %01100110
 .db %01111110
 .db %01100110
 .db %01100110
 .db %01100110
 .db %00000000

FontChar_49:
 .db %00111100
 .db %00011000
 .db %00011000
 .db %00011000
 .db %00011000
 .db %00011000
 .db %00111100
 .db %00000000

FontChar_4A:
 .db %00001111
 .db %00000110
 .db %00000110
 .db %00000110
 .db %01100110
 .db %01100110
 .db %00111100
 .db %00000000

FontChar_4B:
 .db %01110011
 .db %00110011
 .db %00110110
 .db %00111100
 .db %00110110
 .db %00110011
 .db %01110011
 .db %00000000

FontChar_4C:
 .db %01111000
 .db %00110000
 .db %00110000
 .db %00110000
 .db %00110001
 .db %00110011
 .db %01111111
 .db %00000000

FontChar_4D:
 .db %01100011
 .db %01110111
 .db %01111111
 .db %01111111
 .db %01101011
 .db %01100011
 .db %01100011
 .db %00000000

FontChar_4E:
 .db %01100011
 .db %01110011
 .db %01111011
 .db %01101111
 .db %01100111
 .db %01100011
 .db %01100011
 .db %00000000

FontChar_4F:
 .db %00111110
 .db %01100011
 .db %01100011
 .db %01100011
 .db %01100011
 .db %01100011
 .db %00111110
 .db %00000000

FontChar_50:
 .db %01111110
 .db %00110011
 .db %00110011
 .db %00111110
 .db %00110000
 .db %00110000
 .db %01111000
 .db %00000000

FontChar_51:
 .db %00111110
 .db %01100011
 .db %01100011
 .db %01100011
 .db %01100011
 .db %01101111
 .db %00111110
 .db %00000011

FontChar_52:
 .db %01111110
 .db %00110011
 .db %00110011
 .db %00111110
 .db %00110110
 .db %00110011
 .db %01110011
 .db %00000000

FontChar_53:
 .db %00111100
 .db %01100110
 .db %01110000
 .db %00111000
 .db %00001110
 .db %01100110
 .db %00111100
 .db %00000000

FontChar_54:
 .db %01111110
 .db %01011010
 .db %00011000
 .db %00011000
 .db %00011000
 .db %00011000
 .db %00111100
 .db %00000000

FontChar_55:
 .db %01100011
 .db %01100011
 .db %01100011
 .db %01100011
 .db %01100011
 .db %01100011
 .db %00111110
 .db %00000000

FontChar_56:
 .db %01100110
 .db %01100110
 .db %01100110
 .db %01100110
 .db %01100110
 .db %00111100
 .db %00011000
 .db %00000000

FontChar_57:
 .db %01100011
 .db %01100011
 .db %01100011
 .db %01101011
 .db %01111111
 .db %01110111
 .db %01100011
 .db %00000000

FontChar_58:
 .db %01100011
 .db %01100011
 .db %00110110
 .db %00011100
 .db %00011100
 .db %00110110
 .db %01100011
 .db %00000000

FontChar_59:
 .db %01100110
 .db %01100110
 .db %01100110
 .db %00111100
 .db %00011000
 .db %00011000
 .db %00111100
 .db %00000000

FontChar_5A:
 .db %01111111
 .db %01100011
 .db %01000110
 .db %00001100
 .db %00011001
 .db %00110011
 .db %01111111
 .db %00000000

FontChar_5B:
 .db %00111100
 .db %00110000
 .db %00110000
 .db %00110000
 .db %00110000
 .db %00110000
 .db %00111100
 .db %00000000

FontChar_5C:
 .db %01100000
 .db %00110000
 .db %00011000
 .db %00001100
 .db %00000110
 .db %00000011
 .db %00000001
 .db %00000000

FontChar_5D:
 .db %00111100
 .db %00001100
 .db %00001100
 .db %00001100
 .db %00001100
 .db %00001100
 .db %00111100
 .db %00000000

FontChar_5E:
 .db %00001000
 .db %00011100
 .db %00110110
 .db %00000000
 .db %00000000
 .db %00000000
 .db %00000000
 .db %00000000

FontChar_5F:
 .db %00000000
 .db %00000000
 .db %00000000
 .db %00000000
 .db %00000000
 .db %00000000
 .db %00000000
 .db %11111111

FontChar_60:
 .db %00011000
 .db %00011000
 .db %00001100
 .db %00000000
 .db %00000000
 .db %00000000
 .db %00000000
 .db %00000000

FontChar_61:
 .db %00000000
 .db %00000000
 .db %00111100
 .db %00000110
 .db %00111110
 .db %01100110
 .db %00111011
 .db %00000000

FontChar_62:
 .db %01110000
 .db %00110000
 .db %00111110
 .db %00110011
 .db %00110011
 .db %00110011
 .db %01101110
 .db %00000000

FontChar_63:
 .db %00000000
 .db %00000000
 .db %00111100
 .db %01100110
 .db %01100000
 .db %01100110
 .db %00111100
 .db %00000000

FontChar_64:
 .db %00001110
 .db %00000110
 .db %00111110
 .db %01100110
 .db %01100110
 .db %01100110
 .db %00111011
 .db %00000000

FontChar_65:
 .db %00000000
 .db %00000000
 .db %00111100
 .db %01100110
 .db %01111110
 .db %01100000
 .db %00111100
 .db %00000000

FontChar_66:
 .db %00011100
 .db %00110110
 .db %00110000
 .db %01111000
 .db %00110000
 .db %00110000
 .db %01111000
 .db %00000000

FontChar_67:
 .db %00000000
 .db %00000000
 .db %00111011
 .db %01100110
 .db %01100110
 .db %00111110
 .db %00000110
 .db %01111100

FontChar_68:
 .db %01110000
 .db %00110000
 .db %00110110
 .db %00111011
 .db %00110011
 .db %00110011
 .db %01110011
 .db %00000000

FontChar_69:
 .db %00011000
 .db %00000000
 .db %00111000
 .db %00011000
 .db %00011000
 .db %00011000
 .db %00111100
 .db %00000000

FontChar_6A:
 .db %00000110
 .db %00000000
 .db %00001110
 .db %00000110
 .db %00000110
 .db %01100110
 .db %01100110
 .db %00111100

FontChar_6B:
 .db %01110000
 .db %00110000
 .db %00110011
 .db %00110110
 .db %00111100
 .db %00110110
 .db %01110011
 .db %00000000

FontChar_6C:
 .db %00111000
 .db %00011000
 .db %00011000
 .db %00011000
 .db %00011000
 .db %00011000
 .db %00111100
 .db %00000000

FontChar_6D:
 .db %00000000
 .db %00000000
 .db %11100110
 .db %01111111
 .db %01111111
 .db %01101011
 .db %01100011
 .db %00000000

FontChar_6E:
 .db %00000000
 .db %00000000
 .db %01101110
 .db %00110011
 .db %00110011
 .db %00110011
 .db %00110011
 .db %00000000

FontChar_6F:
 .db %00000000
 .db %00000000
 .db %00111100
 .db %01100110
 .db %01100110
 .db %01100110
 .db %00111100
 .db %00000000

FontChar_70:
 .db %00000000
 .db %00000000
 .db %01101110
 .db %00110011
 .db %00110011
 .db %00111110
 .db %00110000
 .db %01111000

FontChar_71:
 .db %00000000
 .db %00000000
 .db %00111011
 .db %01100110
 .db %01100110
 .db %00111110
 .db %00000110
 .db %00001111

FontChar_72:
 .db %00000000
 .db %00000000
 .db %01101110
 .db %00111011
 .db %00110011
 .db %00110000
 .db %01111000
 .db %00000000

FontChar_73:
 .db %00000000
 .db %00000000
 .db %00111110
 .db %01100000
 .db %00111100
 .db %00000110
 .db %01111100
 .db %00000000

FontChar_74:
 .db %00010000
 .db %00110000
 .db %01111100
 .db %00110000
 .db %00110000
 .db %00110110
 .db %00011100
 .db %00000000

FontChar_75:
 .db %00000000
 .db %00000000
 .db %01100110
 .db %01100110
 .db %01100110
 .db %01100110
 .db %00111011
 .db %00000000

FontChar_76:
 .db %00000000
 .db %00000000
 .db %01100110
 .db %01100110
 .db %01100110
 .db %00111100
 .db %00011000
 .db %00000000

FontChar_77:
 .db %00000000
 .db %00000000
 .db %01100011
 .db %01101011
 .db %01111111
 .db %01111111
 .db %00110110
 .db %00000000

FontChar_78:
 .db %00000000
 .db %00000000
 .db %01100011
 .db %00110110
 .db %00011100
 .db %00110110
 .db %01100011
 .db %00000000

FontChar_79:
 .db %00000000
 .db %00000000
 .db %01100110
 .db %01100110
 .db %01100110
 .db %00111110
 .db %00000110
 .db %01111100

FontChar_7A:
 .db %00000000
 .db %00000000
 .db %01111110
 .db %01001100
 .db %00011000
 .db %00110010
 .db %01111110
 .db %00000000

FontChar_7B:
 .db %00001110
 .db %00011000
 .db %00011000
 .db %01110000
 .db %00011000
 .db %00011000
 .db %00001110
 .db %00000000

FontChar_7C:
 .db %00011000
 .db %00011000
 .db %00011000
 .db %00000000
 .db %00011000
 .db %00011000
 .db %00011000
 .db %00000000

FontChar_7D:
 .db %01110000
 .db %00011000
 .db %00011000
 .db %00001110
 .db %00011000
 .db %00011000
 .db %01110000
 .db %00000000

FontChar_7E:
 .db %00111011
 .db %01101110
 .db %00000000
 .db %00000000
 .db %00000000
 .db %00000000
 .db %00000000
 .db %00000000

FontChar_7F:
 .db %00000000
 .db %00001000
 .db %00011100
 .db %00110110
 .db %01100011
 .db %01100011
 .db %01111111
 .db %00000000

; map format:
;   byte[7][32] : tile data
;   byte        : number of tiles
;   byte        : starting level time
;   word        : player start (x,y)
;   byte        : starting screen position

Maps:

Map1:
 .db 01,01,01,01,01,05,05,24,24,24,05,05,05,05,03,13,03,05,05,05,05,05,00,05,05,05,05,15,15,15,15,15
 .db 01,20,01,01,01,05,05,05,05,05,05,05,05,05,03,03,03,05,13,05,05,05,00,05,05,05,05,15,15,15,20,15
 .db 01,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,15
 .db 01,05,05,05,05,05,05,13,05,00,00,00,05,05,05,05,05,05,05,05,05,05,05,05,04,05,05,05,05,05,05,15
 .db 01,05,05,05,05,05,05,05,05,05,05,05,05,05,05,00,00,00,05,05,05,05,05,05,05,05,05,05,05,05,05,15
 .db 01,28,01,01,01,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,05,03,03,03,05,05,05,15,15,15,28,15
 .db 01,01,01,13,01,05,05,16,16,16,05,05,05,05,05,05,13,05,05,05,05,03,13,03,05,05,05,15,15,15,15,15
 .db 158, 99, 40,32, 20<<3

.end
